#!/usr/bin/env python

from MoinMessage import *
from email.mime.text import MIMEText
from email.parser import Parser
import sys

def decrypt(message):

    "Decrypt 'message' if necessary."

    if is_encrypted(message):
        text = gpg.decryptMessage(message)
        return Parser().parsestr(text)
    else:
        return message

def verify(message):

    "Verify 'message' if possible."

    if is_signed(message):
        return gpg.verifyMessage(message)
    else:
        return None, None, message

if __name__ == "__main__":
    try:
        signer = sys.argv[1]
        recipient = sys.argv[2]
        url = sys.argv[3] + "?action=FetchMessages"
        args = sys.argv[4:]
    except IndexError:
        args = None

    if not args:
        print >>sys.stderr, "Need a signer, recipient, URL and some commands as arguments to this program."
        sys.exit(1)

    message = Message()
    parts = []

    part = MIMEText("\n".join(args), "plain", sys.stdin.encoding)
    message.add_update(part)

    # Sign and encrypt the request.

    email_message = message.get_payload()
    gpg = GPG()

    try:
        signed_message = gpg.signMessage(email_message, signer)
        encrypted_message = gpg.encryptMessage(signed_message, recipient)
        resp = sendMessageOpener(encrypted_message, url)
        mailbox = Parser().parse(resp)

        # Decrypt and verify the response.

        mailbox = decrypt(mailbox)
        fingerprint, identity, content = verify(mailbox)

    except MoinMessageError, exc:
        print >>sys.stderr, exc

    # Indicate the signature details of the response.

    if fingerprint and identity:
        print >>sys.stderr, "Messages signed by", fingerprint, identity
    else:
        print >>sys.stderr, "Message not signed!"

    print >>sys.stderr

    # Show the retrieved messages.

    if content.is_multipart():
        for part in content.get_payload():

            try:
                part = decrypt(part)
                fingerprint, identity, part = verify(part)

            except MoinMessageError, exc:
                print >>sys.stderr, "Message not shown."
                print >>sys.stderr, exc
                continue

            if fingerprint and identity:
                print >>sys.stderr, "Part signed by", fingerprint, identity
            print part
    else:
        print content

# vim: tabstop=4 expandtab shiftwidth=4
